//
//  SITBuilding.h
//  SitumSDK
//
//  Created by A Barros on 1/3/17.
//  Copyright © 2017 Situm. All rights reserved.
//

#import <Foundation/Foundation.h>
#import <CoreLocation/CoreLocation.h>

#import "SITAngle.h"
#import "SITDimensions.h"
#import "SITResource.h"
#import "SITURL.h"
#import "SITBounds.h"

/**
 Provides information about the places where indoor location is supported.
 */
@interface SITBuilding : SITResource

#pragma mark - Initializers

/** Initializer method
 @param createdAt Creation date
 @param updatedAt Last updated date
 @param customFields Key-value pairs that can be used to store information
 @param userIdentifier Identifier of the user who creates the building
 @param name Name if the building
 @param center Cartesian coordinates of the center of the building
 @param info String containing additional information
 @param dimensions Structure containing the widht and height of the building
 @param rotation Rotation angle of the building's base, relative to the west-east axis,
 increasing in counter-clockwise, being 0 the west-east axis.
 @param pictureURL URL of the building image
 @param pictureThumbURL URL of the thumbnail building image
 @return `SITBuilding`
 */
- (instancetype)initWithIdentifier:(NSString *)identifier
                         createdAt:(NSDate *)createdAt
                         updatedAt:(NSDate *)updatedAt
                      customFields:(NSDictionary *)customFields
                    userIdentifier:(NSString *)userIdentifier
                              name:(NSString *)name
                            center:(CLLocationCoordinate2D)center
                              info:(NSString *)info
                        dimensions:(SITDimensions *)dimensions
                          rotation:(SITAngle *)rotation
                        pictureURL:(SITURL *)pictureURL
                   pictureThumbURL:(SITURL *)pictureThumbURL;

#pragma mark - Properties

/**
 Identifier of the user who created the building
 */
@property(nonatomic, strong) NSString *userIdentifier;

/**
 *  Name of the building.
 */
@property(nonatomic, strong) NSString *name;

/**
 *  Address of the building.
 */
@property(nonatomic, strong) NSString *address;

/**
 *  Detailed information in HTML format
 */
@property(nonatomic, strong) NSString *infoHTML;

/**
 Rotation angle of the building's base, relative to the west-east axis,
 increasing in counter-clockwise, being 0 the west-east axis.
 */
@property(nonatomic, strong) SITAngle *rotation;

/**
 * URL of the building image
 */
@property(nonatomic, strong) SITURL *pictureURL;

/**
 * URL of the thumbnail building image
 */
@property(nonatomic, strong) SITURL *pictureThumbURL; // nail TODO: Complete names??

#pragma mark - Methods

/**
 * Compute corners of this building, without rotation, in earth coordinates.
 *
 * @return `SITBounds`
 */
- (SITBounds)bounds;

/**
 * Compute corners of this building, with rotation, in earth coordinates.
 *
 * @return `SITBounds`
 */
- (SITBounds)rotatedBounds;


/**
 Size of the building

 @return `SITDimensions` structure containing the widht and height of the building
 */
- (SITDimensions *)dimensions;

/**
 Geographical coordinate of the building
 
 @return `CLLocationCoordinate2D`
 */
- (CLLocationCoordinate2D)center;


@end
